#!/usr/bin/perl

use strict;
use warnings;

open(TXT, $ARGV[0].".txt");
open(CDG, $ARGV[0].".cdg");
my @sentences = <CDG>;
my $plain = "";

foreach my $current_sentence (@sentences) {
  #$current_sentence = $sentences[substr($sentence_id, 1) -1]; #first line (index 0) of tokenized file is s1
  $current_sentence =~ s/'s\d+' : //; ## remove the sentence label, e.g. 's1' : Warum, mssen, --> Warum, mssen,
  $current_sentence = substr($current_sentence, 0, -2); # remove the semicolon from the end
  # Split into tokens by delimiter ", " (comma followed by space), but regard commas in quoted tokens such as 'a, b'
  # Temporarily replace those by a one-byte unicode character that is not in the latin1 set (choose ones from cp-1252)
  # Use an opening and a closing placeholder, to prevent recognizing a comma in between in e.g.: 'first-special', '?'
  # The quote must be either the beginning of a line, or must be preceded by a character that is not a backslash (it must be
  ## non-escaped. However, an escaped backslash can occur in front of it, so do allow an even number of backslashes. 
  while ($current_sentence =~ s/(^|[^\\]|(\\\\)+?)'/$1\x{2039}/) {
         $current_sentence =~ s/([^\\]|(\\\\)+?)'/$1\x{203a}/; # I assume an even number of non-escaped quotes
  }
  # if a comma followed by a space occurs within a quoted context, replace the space by a placeholder
  # as lazy as possible using *? in order not to match too large contexts 
  while ($current_sentence =~ s/(\x{2039}[^\x{2039}\x{203a}]*?), ([^\x{2039}\x{203a}]*?\x{203a})/$1,\x{2022}$2/g) {}
  my @current_tokens = split(/\, /, $current_sentence); # split into tokens by delimiter ", " (comma followed by space)
  foreach my $t (@current_tokens) {
    $t =~ s/^\x{2039}(.*)\x{203a}$/$1/; # remove single quotes that are around some tokens, e.g. ',' --> ,
    $t =~ s/\\('|"|\\)/$1/g; # decode escaped ' and " and \
    $t =~ tr/\x{2022}/ /; #change back spaces
    $t =~ s/\s+//g; ## remove whitespace
    while (length($t) > length($plain)) {
      my $plain_line = <TXT>;
      if (!$plain_line) { last; }
      $plain_line =~ s/\s+//g; ## remove whitespace from contents
       $plain_line =~ tr/\x{ab}\x{bb}/""/; # change double left- and right-pointing quotes to normal double quotes for matching, 
                                           # because tokenizer does this too. otherwise, some " in tokens might not match the input text
      $plain .= $plain_line;
    }
    my $startpos = index($plain, $t);
    ## IF PLAIN TEXT MAY CONTAIN ADDITIONAL CHARACTERS: MAPPING TO TOKENIZED TEXT POSSIBLE USING DELETION (NOT REPLACEMENT OR ADDITION)
    #if ($startpos > 0) { #matching not succeeded.
    #  warn "token $t \ndoes not match \n$plain \n\n";
    #}
    if ($startpos == -1) { #matching not succeeded.
      die "token $t \ncannot be found in \n$plain \n\n";
    }
    else {
      $plain = substr($plain, $startpos+length($t));
    }
  }
}

print "Matching ok.\n";
